import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  scaleOrdinal,
  schemeCategory10,
  schemeYlOrRd,
  format
} from 'd3';
import { colorLegend } from './colorLegend';
import { loadData } from './loadData';
import { scatterplot } from './scatterplot';

const colorScale = scaleOrdinal();
const colorValue = d => d.artist;
var legend;
var selectedArtist;
var data;

const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');
const margin = { top: 60, right: 30, bottom: 88, left: 200 };
const innerWidth = width - margin.left - margin.right;
const innerHeight = height - margin.top - margin.bottom;
const title = 'Plays in 2018 by Genre';

const scatterG = svg.append('g')
  .attr('transform', `translate(${margin.left},${margin.top})`);
const colorLegendG = svg.append('g')
  .attr('transform', `translate(20, 10)`);

  svg.append('text')
    .attr('class', 'title')
    .attr('x', (width + margin.left) / 2)
    .attr('y', 45)
    .text(title);

loadData('https://vizhub.com/OxfordComma/datasets/output-with-genre-2018.csv').then(processedData => {
  data = processedData.byWeekPlaysGenre;
  legend = processedData.sortedGenres;
  render();
});

const onClick = d => {
  selectedArtist = d;
  render(); 
};

const render = () => {
   
  colorScale
    .domain(legend)
    .range(schemeCategory10); 
  
  colorLegendG.call(colorLegend, {
    colorScale,
    circleRadius: 6,
    spacing: 15,
    textOffset: 12,
    backgroundRectWidth: 135,
    onClick,
    selectedArtist
  });
  
  scatterG.call(scatterplot, {
    data,
    legend,
    colorScale,
    colorValue,
    selectedArtist,
    innerWidth,
    innerHeight,
    circleRadius: 3
  });
};