// Mouseover line adapted from here
// https://bl.ocks.org/larsenmtl/e3b8b7c2ca4787f77d78f58d41c3da91

import { 
  select, 
  scaleTime, 
  scaleLinear, 
  extent,
  axisLeft,
  axisBottom,
  format,
  nest,
  line,
  area,
  curveBasis,
  mouse,
  stack,
  max,
  sum
} from 'd3';

export const scatterplot = (selection, props) => {
  const {
    data,
    legend,
    colorScale,
    colorValue,
    selectedArtist,
    innerWidth,
    innerHeight,
    circleRadius
  } = props;
    
  const g = selection.selectAll('.container').data([null]);
  const gEnter = g.enter()
    .append('g')
      .attr('class', 'container');
 
  //const xValue = d => d.week;

  const xAxisLabel = 'Week';
  const yAxisLabel = 'Plays'; 
  
  // X-scale and x-axis
  const xScale = scaleLinear()
  	.domain(extent(data, d => d.week))
    .range([0, innerWidth])
  
  const xAxis = axisBottom(xScale)
    .ticks(26)
    .tickSize(-innerHeight)
    .tickPadding(15);
  
  // From https://vizhub.com/curran/501f3fe24cfb4e6785ac75008b530a83
  const xAxisG = g.select('.x-axis');
  const xAxisGEnter = gEnter
    .append('g').attr('class', 'x-axis');
  
  xAxisGEnter
    .merge(xAxisG)
      .call(xAxis)
  	  .attr('transform', `translate(0,${innerHeight})`)
      .selectAll('.domain').remove()
  
  xAxisGEnter.append('text')
      .attr('class', 'axis-label')
      .attr('y', 50)
      .attr('x', innerWidth / 2)
      .attr('fill', 'black')
      .text(xAxisLabel);
	
  // Y-axis and scale
  const yScale = scaleLinear()
    .domain([0, 
             //selectedArtist ? 
             //max(data.map(d => d[selectedArtist])) : 
             max(data.map(d => sum(Object.values(d))))])
    .range([innerHeight, 0])
    .nice();  
  
  const yAxisTickFormat = number =>
    format('.2s')(number)
      .replace('.0', '');
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(5)
    .tickFormat(yAxisTickFormat);
  
  const yAxisG = g.select('.y-axis');
  const yAxisGEnter = gEnter
    .append('g')
      .attr('class', 'y-axis');
  
  yAxisGEnter
    .merge(yAxisG)
  		.transition().duration(200)
      .call(yAxis);
  
  yAxisGEnter.merge(yAxisG)
      .selectAll('.domain').remove();
  
  yAxisGEnter.append('text')
    .attr('class', 'axis-label')
    .attr('y', -35)
    .attr('x', -innerHeight / 2)
    .attr('fill', 'black')
    .attr('transform', `rotate(-90)`)
    .attr('text-anchor', 'middle')
    .text(yAxisLabel);
  
  var stack = d3.stack(data)
  	.keys(legend);
  var series = stack(data);
  //console.log(data)
  
  const areaGenerator = area()
    .x(d => xScale(d.data.week))
    .y0(d => {
      //console.log(d)
      return yScale(selectedArtist && (d.artist == selectedArtist) ? 0 : d[0])})
    .y1(d => yScale(selectedArtist && (d.artist == selectedArtist) ? d[1] - d[0] : d[1]))
    .curve(curveBasis);
  
  const lastYValue = d =>
    yValue(d.values[d.values.length - 1]);
  
  const lines = selection.selectAll('.line-path').data(series);
  const linesEnter = lines.enter().append('path')
      .attr('class', 'line-path') 
  		.attr('fill', d => colorScale(d.key))
  		.attr('stroke', 'black')
    	
  lines.merge(linesEnter)
  	.transition()
    	.duration(200)
  		.attr('d', areaGenerator)
    	.attr('opacity', d => (!selectedArtist || d.key === selectedArtist) ? 1 : 0)
      .attr('stroke-width', d => (selectedArtist || d.key === selectedArtist) ? 0 : 0);
  
  // Mouse code 
  /*
  const mouseLine = gEnter.append('path')
      .attr('class', 'mouse-line')
      .style('stroke', 'black')
      .style('stroke-width', '1px')
      .style('opacity', '0');
  
	gEnter.append('svg:rect') // append a rect to catch mouse movements on canvas
    .attr('width', innerWidth) // can't catch mouse events on a g element
    .attr('height', innerHeight)
    .attr('fill', 'none')
    .attr('pointer-events', 'all')
    .on('mouseout', function() { // on mouse out hide line, circles and text
      d3.select('.mouse-line').style('opacity', '0');
    })
    .on('mouseover', function() { // on mouse in show line, circles and text
      d3.select('.mouse-line').style('opacity', '1');
    })
    .on('mousemove', function() { // mouse moving over canvas
      var mouse = d3.mouse(this);
      
    d3.select('.mouse-line')
        .attr('d', function() {
          var d = 'M' + mouse[0] + ',' + innerHeight;
          d += ' ' + mouse[0] + ',' + 0;
          return d;
        });
    });
    */
};
